//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
package grace.log;

import java.util.Date;
import java.util.StringTokenizer;
import java.util.Vector;


/**
 * This class captures all of the data needed to record a programmatic
 * entry to one of the log Handlers.  It is mainly a bunch of data but
 * it has a little functionality.  The grace.log.Log class builds an
 * Event each time a call is made to one of its static functions:
 * <code>log()</code>, <code>error()</code>, <code>warning()</code>,
 * <code>notice()</code>, or <code>trace()</code>.  Each of these
 * functions calls the Handler.log(Event) function.
 *
 * @author Christopher Bladon
 **/
public class Event implements java.io.Serializable {
    public static final String rcsid = "$Id: Event.java,v 1.6 1999/10/11 00:09:59 cbladon Exp $";

    String		hostname;
    String		virtualMachineName;
    String		type;
    Date		time;
    long		number;
    StackTrace		position;
    String		message;
    Object		object;
    Vector		threadNames;
    private boolean	objectGiven;	// In case object is null

    private static long		nextNumber = 1;


    public Event(String type) {
	this.type = type;
	time = new Date();
	number = nextNumber++;
	threadNames = createThreadNames();
    }

    public void setHostname(String hostname) { this.hostname = hostname; }
    public void setVirtualMachineName(String name) {
	this.virtualMachineName = name;
    }
    public void setPosition(StackTrace position) {
	this.position = position;
    }
    public void setMessage(String message) { this.message = message; }
    public void setObject(Object object) {
	this.object = object;
	objectGiven = true;
    }

    public String getHostname() { return hostname; }
    public String getVirtualMachineName() { return virtualMachineName; }
    public String getType() { return type; }
    public Date getTime() { return time; }
    public long getNumber() { return number; }
    public StackTrace getPosition() { return position; }
    public String getMessage() { return message; }
    public Object getObject() { return object; }
    public boolean objectWasGiven() { return objectGiven; }



    protected void appendGroupName(Vector names, ThreadGroup group)
    {
	ThreadGroup parent = group.getParent();
	if (parent != null) appendGroupName(names, parent);
	
	names.addElement(group.getName());
    }
    
    protected Vector createThreadNames()
    {
	Vector names = new Vector();
	Thread current = Thread.currentThread();
	ThreadGroup parent = current.getThreadGroup();
	if (parent != null) appendGroupName(names, parent);

	names.addElement(current.getName());

	return names;
    }
}
    
