//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
package grace.log;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.Statement;


/**
 * This handles events by writing them to a JDBC database.  The
 * following properties are used:
 *
 * <p>Properties:
 * <p><pre>
 *     log.handler.<i>name</i>.url = jdbc:idb:idb.prp
 *     log.handler.<i>name</i>.table = Log
 *     log.handler.<i>name</i>.format =
 *         "'%t', %n, '%e', '%h', '%c', '%f', '%l', '%m', '%(%j)o'"
 *     log.handler.<i>name</i>.username = ""
 *     log.handler.<i>name</i>.password = ""
 *     log.handler.<i>name</i>.columns = "(t, number, type, ...)"
 * </pre>
 *
 * <p>This handler assumes that the database and table are already
 * created on the target database.  Using the default column and
 * format, this handler will write entry using the following SQL:
 *
 * <p><pre> insert into Log values ('12/31/99 23:59:59', 1001,
 * 'error', 'thread-1', 'MyClass', 'myFunc', '129', 'problem with
 * something', 'String s = "a string"'); </pre>
 *
 * <p>In other words this writes the log entry without assuming the
 * name of the columns in the database but it assumes the order and
 * type.  Of course this can be changed by simple specifying the
 * '.table', '.columns', and/or '.format' properties.
 *
 * <p>Here is the schema that was used to test this Handler using the
 * postgres database.
 * <pre>
 * 	CREATE TABLE Log (
 *		eventtime   datetime,
 *  		number      int,
 *		type        varchar(32),
 *  		thread      varchar(64),
 *  		class       varchar(64),
 *  		function    varchar(64),
 *  		line        varchar(8),
 *  		message     varchar(256),
 *  		object      varchar(256));
 * </pre>
 *
 * <p>But notice that using the default properties, the column names
 * don't really matter - just the order in which they appear in the
 * schema.
 *
 * <P>Properties:
 *
 * <ul>
 * <li>log.handler.<i>name</i>.url=jdbc:<i>sub-protocol://host/db</i>
 * <li>log.handler.<i>name</i>.username=<i>username</i>
 * <li>log.handler.<i>name</i>.password=<i>password</i>
 * <li>log.handler.<i>name</i>.table=<i>table-name</i>
 * <li>log.handler.<i>name</i>.columns=<i>comma-separated-column-list</i>
 * <li>log.handler.<i>name</i>.format=<i>format-string</i> // EventFormat
 * </ul>
 *
 **/
public class JDBCHandler
    implements Handler
{
    private Connection connection;
    private EventFormat format;
    private String table;
    private String columns;
    
    public JDBCHandler(String prefix, String name)
	throws java.rmi.RemoteException
    {
	String propertyPrefix = prefix + name;

	try {
	    connection = DriverManager.getConnection (
		System.getProperty(
		    propertyPrefix + ".url",
		    "jdbc:idb:idb.prp"),
		System.getProperty(propertyPrefix + ".username", ""),
		System.getProperty(propertyPrefix + ".password", ""));

	    DriverManager.setLogStream(null);
	}
	catch (Exception e) {
	    e.printStackTrace();
	}

	table = System.getProperty(propertyPrefix + ".table", "Log");

	columns = System.getProperty(propertyPrefix + ".columns", "");
	if (!columns.equals("") && !columns.startsWith("(")) {
	    columns = '(' + columns + ')';
	}

	format = new EventFormat(
	    System.getProperty(
		propertyPrefix + ".format",
		"'%t', %n, '%e', '%h', '%c', '%f', '%l', '%m', '%(Java)o'"));
    }


    public void handle(Event event) {
	try {
	    Statement statement = connection.createStatement ();

	    String formattedEvent = format.format(event);
	    if (formattedEvent.length() > 256) {
		formattedEvent = formattedEvent.substring(0, 256) + "'";
	    }
	    
	    int numRows = statement.executeUpdate(
		"insert into " + table +
		" " + columns +
		" values (" + formattedEvent + ")");
	}
	catch (Exception e) {
	    e.printStackTrace();
	}
    }
}
