//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
package grace.log;

/**
 * This class allows the user of JavaLog to use existing
 * System.out.print statements in legacy code, but route the output of
 * those statements through the JavaLog formatting system.  This is
 * accomplished by installing one instance of this class in place of
 * the System output or error stream using the
 * <code>System.setOut()</code> or <code>System.setErr()</code>
 * functions.
 *
 * <P>The best way to use the class is to use the static functions
 * <code>install, installOut, or installErr</code>.  These will
 * install PrintCatchers for out and err, only out, or only err,
 * respectively.
 **/
public class PrintCatcher extends java.io.PrintStream {
    public static final String rcsid = "$Id: PrintCatcher.java,v 1.2 1999/10/11 00:09:45 cbladon Exp $";

    private java.io.PrintStream original;
    private String eventType = Log.NOTICE;
    private boolean bufferPrints = true;
    private StringBuffer buffer = new StringBuffer();

    /**
     * Installs a PrintCatcher over the standard output and one over
     * the standard error.
     *
     * @see installOut
     * @see installErr
     **/
    public static void install() {
	installOut();
	installErr();
    }

    /**
     * Replaces any PrintCatchers previously installed as the standard
     * output and standard error with original standard out and error.
     **/
    public static void uninstall() {
	if (System.out instanceof PrintCatcher)
	    System.setOut(((PrintCatcher) System.out).getOriginalStream());

	if (System.err instanceof PrintCatcher)
	    System.setErr(((PrintCatcher) System.err).getOriginalStream());
    }

    /**
     * Installs a new PrintCatcher on the standard out using
     * Log.NOTICE as the event type.
     **/
    public static void installOut() { installOut(Log.NOTICE); }

    /**
     * Installs a new PrintCatcher on the standard out using the given
     * event type.
     **/
    public static void installOut(String eventType) {
	if (!(System.out instanceof PrintCatcher)) {
	    Log.internal("installed PrintCatcher on System.out");
	    System.setOut(new PrintCatcher(eventType, System.out));
	}
    }

    /**
     * Installs a new PrintCatcher on the standard err using
     * Log.ERROR as the event type.
     **/
    public static void installErr() { installErr(Log.ERROR); }

    /**
     * Installs a new PrintCatcher on the standard err using the given
     * event type.
     **/
    public static void installErr(String eventType) {
	if (!(System.err instanceof PrintCatcher)) {
	    Log.internal("installed PrintCatcher on System.err");
	    System.setErr(new PrintCatcher(eventType, System.err));
	}
    }
	
    
    public PrintCatcher() {
	super(System.out);
	original = System.out;
    }
    
    public PrintCatcher(String eventType) {
	this(eventType, System.out);
	original = System.out;
    }
    
    public PrintCatcher(String eventType, java.io.PrintStream stream) {
	super(stream);
	original = stream;
    }

    public java.io.PrintStream getOriginalStream() { return original; }

    public void setBufferedPrints(boolean buffered) { bufferPrints = buffered; }
    
    public void print(char v) { print("" + v, 2); }
    public void println(char v) { println("" + v, 2); }
    public void print(char[] v) { print("" + v, 2); }
    public void println(char[] v) { println("" + v, 2); }
    public void print(boolean v) { print("" + v, 2); }
    public void println(boolean v) { println("" + v, 2); }
    public void print(int v) { print("" + v, 2); }
    public void println(int v) { println("" + v, 2); }
    public void print(long v) { print("" + v, 2); }
    public void println(long v) { println("" + v, 2); }
    public void print(float v) { print("" + v, 2); }
    public void println(float v) { println("" + v, 2); }
    public void print(double v) { print("" + v, 2); }
    public void println(double v) { println("" + v, 2); }
    public void print(String v) { print(v, 2); }
    public void println(String v) { println(v, 2); }

    protected void print(String message, int level) {
	if (bufferPrints) { buffer.append(message); }
	else {
	    Event event = new Event(eventType);
	    event.setMessage(message);
	    event.setPosition(new StackTrace(level));
	
	    Log.getInstance().distribute(event);
	}
    }

    protected void println(String message, int level) {
	if (bufferPrints) {
	    buffer.append(message);
	    message = buffer.toString();
	    buffer.setLength(0);
	}
	    
	Event event = new Event(eventType);
	event.setMessage(message);
	event.setPosition(new StackTrace(level));
	
	Log.getInstance().distribute(event);
    }

    public void print(Object object) {
	Event event = new Event(eventType);
	event.setObject(object);
	event.setPosition(new StackTrace(1));
	
	Log.getInstance().distribute(event);
    }

    public void print(Exception exception) {
	Event event = new Event(Log.ERROR);
	event.setObject(exception);
	event.setPosition(new StackTrace(exception));
	
	Log.getInstance().distribute(event);
    }
}
