//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
package grace.log;

/**
 * This class is a hack to allow the Handler class to break up a stack
 * trace into each function call so that only the top most function
 * can be displayed in the log.  Java should provide a more convienent
 * way to do this but it doesn't.
 **/
public class Writer extends java.io.PrintWriter {
    public static final String rcsid = "$Id: Writer.java,v 1.7 1999/10/11 00:10:01 cbladon Exp $";

    private int lineNum = 0;
    private String line = "";
    private int desiredLineNum = 3;

    public Writer() { super(new java.io.StringWriter()); }

    protected boolean partOfCallStack(String s) {
	int i = 0;
	while (i < s.length() && !Character.isLetter(s.charAt(i))) ++i;
	return i < s.length() - 1 && s.charAt(i) == 'a'
	    && s.charAt(i + 1) == 't';
    }
    protected boolean partOfCallStack(char[] s) {
	int i = 0;
	while (i < s.length && !Character.isLetter(s[i])) ++i;
	return i < s.length - 1 && s[i] == 'a' && s[i + 1] == 't';
    }
    
    public void print(char[] s) {
	if (partOfCallStack(s)) {
	    if (++lineNum == desiredLineNum) line = new String(s);
	}
    }
    public void println(char[] s) {
	print(s);
    }
    public void write(char[] s) {
	print(s);
    }
    public void write(char[] s, int offset, int length) {
	print(String.copyValueOf(s, offset, length));
    }
    
    public void print(Object o) {
	print(o.toString());
    }
    public void println(Object o) {
	print(o.toString());
    }

    public void print(String s) {
	if (partOfCallStack(s)) {
	    if (++lineNum == desiredLineNum) line = s;
	}
    }
    public void println(String s) {
	print(s);
    }
    public void write(String s) {
	print(s);
    }
    public void write(String s, int offset, int length) {
	print(s.substring(offset, offset + length));
    }

    public void print(boolean b) {
	System.out.println("print(boolean) unhandled");
    }
    public void println(boolean b) { print(b); }

    public void print(char c) {
	System.out.println("print(char) unhandled");
    }
    public void println(char c) { print(c); }

    public void println() {
	System.out.println("println() unhandled");
    }
    
    public String toString() {
	return line.trim();
    }

    public void setDesiredLineNum(int line) {
	desiredLineNum = line;
    }

    public StackTrace result() {
	StackTrace position = new StackTrace();
	position.parse(toString());
	return position;
    }
		
    public void clear() {
	lineNum = 0;
	line = "";
    }
}
