//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
package grace.util;

import java.io.FileInputStream;
import java.util.Vector;
import java.util.Enumeration;

import gnu.regexp.RE;
import gnu.regexp.REMatch;
import gnu.regexp.REException;

/**
 * This class wraps and augments the behaviour of the
 * java.util.Properties class.  It is not inherited from the java
 * class because bonheaded Javasoft made the properties class inherit
 * from Hashtable.  This has many problems which I will not go into
 * here.
 *
 * <P>Anyway, this class allows a more flexible management of
 * properties than the java version and allows a regular expression
 * query mechanism.  Note that this mechanism is a linear search as
 * opposed to the get functions which use the hashtable lookup.
 *
 * <P>All properties loaded into this class are copied from their
 * original list.  This makes it easier to layer properties where
 * different priorities are intended for differernt sources of
 * properties.  Typically, one would like to make default properties
 * from the user's home directory the lowest priority, properties from
 * a local file the next priority, properties from the command line
 * the next, and properties set within the application the highest.
 *
 * <P>This class also manages parameters of an Applet so that users
 * can treat properties and parameters as the same.
 *
 * <P>So, here is a typical use:
 * <pre>
 *	grace.util.Properties properties = new grace.util.Properties();
 *	properties.loadParameters(anApplet);
 *	properties.loadHomeFile(".my-home-props");
 *	properties.loadFile(".my-local-props");
 *	properties.loadSystem();
 *      Vector someProps = properties.getRegexp("log.*");
 * </pre>
 **/
public class Properties {
    java.util.Properties list = new java.util.Properties();
    static java.applet.Applet applet;	// for parameters

    /**
     * A new empty list.
     **/
    public Properties() {}

    /**
     * Constructs a property list that is a copy of the given list.
     **/
    public Properties(grace.util.Properties properties) {
	load(properties);
    }

    /**
     * Constructs a property list that is a copy of the given list.
     **/
    public Properties(java.util.Properties properties) {
	load(properties);
    }

    /**
     * Loads the given properties into this list by coping each
     * property out of the given list and into this list.  Thus, the
     * given properties will override any existing properties in this
     * list.
     **/
    public void load(grace.util.Properties properties) {
	load(properties.list);
    }
    
    /**
     * Loads the given properties into this list by coping each
     * property out of the given list and into this list.  Thus, the
     * given properties will override any existing properties in this
     * list.
     **/
    public void load(java.util.Properties properties) {
	Enumeration names = properties.propertyNames();
	while (names.hasMoreElements()) {
	    String name = (String) names.nextElement();
	    String value = properties.getProperty(name);
	    if (value != null) list.put(name, value);
	}
    }

    /**
     * Remembers the given applet such that all subsequent get()
     * functions on this class will query the applet for a named
     * parameter using the Applet.getParameter(String) call.  The
     * getParameter call is made only when the lookup fails to find
     * the name in this list.  Thus Applet parameters are lower
     * priority than these properties.  Also, none of the functions of
     * this class that return vectors or Enumerations work with
     * Applets parameters.  Don't blame me; blame the Javasoft
     * geniuses.
     *
     * <P>This is static right now because I can't figure out how to
     * handle applets.
     **/
    public static void setApplet(java.applet.Applet a) {
	applet = a;
    }

    /**
     * Loads a copy of the System properties (System.getProperties())
     * into this list.
     **/
    public void loadSystem() {
	load(System.getProperties());
    }

    /**
     * Reads properties from the given file into this list.
     **/
    public boolean loadFile(String propertyFilename) {
	try {
	    list.load(new FileInputStream(propertyFilename));
	    return true;
	}
	catch (java.io.IOException e) { return false; }
    }

    /**
     * Reads properties from the given file into this list where the
     * given file is assumed to be located in the user.home directory.
     **/
    public void loadHomeFile(String basePropertyFilename) {
	String homeDir = System.getProperty("user.home", "");
	String separator = System.getProperty("file.separator");
	loadFile(homeDir + separator + basePropertyFilename);
    }

    /**
     * Sets the value of the property referred to by name.  
     *
     * @param name of property to set
     **/
    public void put(String name, String value) {
	list.put(name, value);
    }

    
    /**
     * Returns the value of the property referred to by name or
     * null if the named property does not exist.
     *
     * @param name of property to get
     * @return value of named property or null property doesn't exist
     **/
    public String get(String name) {
	String result = list.getProperty(name);
	if (result == null && applet != null)
	    result = applet.getParameter(name);

	return result;
    }

    
    /**
     * Returns the value of the property referred to by name or
     * defaultValue if the named property does not exist.
     *
     * <P>Note: This calls grace.util.Properties.get(String);
     *
     * @param name of property to get
     * @param defaultValue to return if named property doesn't exist
     *
     * @return value of named property or defaultValue property doesn't exist
     **/
    public String get(String name, String defaultValue) {
	String result = get(name);
	return result == null ? defaultValue : result;
    }

    
    /**
     * Returns a property by searching linearly for each name in the
     * array of names.  As soon as a name is found, it's value is
     * returned.  This is useful for looking up properties known by
     * potentially many names.
     *
     * <P>Note: This calls grace.util.Properties.get(String[], String);
     *
     * @param names sequential list of names for which to search
     * @return value of named property or null if not existent
     **/
    public String get(String[] names) {
	return get(names, null);
    }

    /**
     * Returns a property value by searching linearly for each name in
     * array of names.  As soon as a name is found, it's value is
     * returned.  This is useful for looking up properties known by
     * potentially many names.  If no properties are found, the
     * defaultValue is returned.
     *
     * <P>Note: This calls grace.util.Properties.get(String);
     *
     * @param names sequential list of names for which to search
     * @param defaultValue to return if no properties are found
     * @return value of named property or defaultValue if not existent
     **/
    public String get(String[] names, String defaultValue) {
	String result;
	for (int i = 0; i < names.length; ++i) {
	    result = get(names[i]);
	    if (result != null) return result;
	}

	return defaultValue;
    }

    /**
     * This returns a vector or properties names that match the given
     * regular expression.
     *
     * <P>These results do not include parameters held by an applet.
     * This is because of general lame design of Javasoft.
     *
     * <P>Note: calls grace.util.Properties.names(RE);
     **/
    public Vector names(String regexp) throws REException {
	return names(new RE(regexp));
    }
	
    /**
     * This returns a vector or names that match the given regular
     * expression.
     *
     * <P>These results do not include parameters held by an applet.
     * This is because of general lame design of Javasoft.
     **/
    public Vector names(RE regexp) {
	Vector names = new Vector();

	Enumeration nameIter = list.propertyNames();
	while (nameIter.hasMoreElements()) {
	    String name = (String) nameIter.nextElement();
	    if (regexp.isMatch(name)) names.addElement(name);
	}

	return names;
    }

    /**
     * Returns a subset of this Properties list narrowed by the given
     * regular expression.
     **/
    public Properties subset(String regexp) throws REException {
	return subset(new RE(regexp));
    }
    
    /**
     * Returns a subset of this Properties list narrowed by the given
     * regular expression.
     **/
    public Properties subset(RE regexp) {
	Properties subset = new Properties();
	Enumeration names = list.propertyNames();
	Enumeration values = list.elements();
	while (names.hasMoreElements()) {
	    String name = (String) names.nextElement();
	    String value = (String) values.nextElement();
	    if (regexp.isMatch(name)) subset.put(name, value);
	}

	return subset;
    }

    /**
     * Returns a subset of this Properties list narrowed by the given
     * regular expression but with the names substituted with the
     * given nameSubstitution.  This allows the caller to build a
     * subset of this Properties list with names that are also
     * narrowed.  The nameSubstitution can contain regalar expression
     * match sequences $0-$9.
     *
     * <P>For example, given a properties list of:
     * <ul>
     * <li>variable = 1
     * <li>variable.x.value = 2
     * <li>variable.y.value = 3
     * <li>variable.z.value = 4
     * <ul>
     *
     * <P>The call <code>subset("variable\\.([^\.]+)\\.value",
     * "$1");</code> would result in a properties list of:
     *
     * <ul>
     * <li>x = 2
     * <li>y = 3
     * <li>z = 4
     * <ul>
     **/
    public Properties subset(RE regexp, String nameSubstitution) {
	Properties subset = new Properties();
	Enumeration names = list.propertyNames();
	while (names.hasMoreElements()) {
	    String name = (String) names.nextElement();
	    String value = get(name);
	    REMatch match = regexp.getMatch(name);
	    if (match != null) {
		subset.put(match.substituteInto(nameSubstitution), value);
	    }
	}

	return subset;
    }

    /**
     * @see subset(RE, String)
     **/
    public Properties subset(String regexp, String nameSubstitution)
	throws REException
    {
	return subset(new RE(regexp), nameSubstitution);
    }
	

    /**
     * Allows interation over all of the names of properties held by
     * this class.  This doesn't include the applet parameters.
     **/
    public Enumeration names() {
	return list.propertyNames();
    }

    /**
     * Allows interation over all of the values of properties held by
     * this class.  This doesn't include the applet parameters.
     **/
    public Enumeration values() {
	return list.elements();
    }

    /**
     * This sets the System properties to the set managed by this
     * class.  It keeps this set of properties segregated from System
     * properties.
     **/
    public void setSystemProperties() {
	System.setProperties(list);
    }

    /**
     * If this set of properties is currently a view of the System
     * properties, then a new interal property collection is created
     * and the System properties are copied into the newly created
     * collection.  If this collection of properties is not mapped to
     * the System properties, then this call does nothing.
     **/
    public void segregateFromSystemProperties() {
	if (list == System.getProperties()) {
	    list = new java.util.Properties();
	    load(System.getProperties());
	}
    }

    /**
     * If this set of properties is currently not a view of the System
     * properties, then the System properties are loaded into this
     * (just to make sure it doesn't wipe out any System properties),
     * and the System properties are set to this collection.  If this
     * collection of properties is already mapped to the System
     * properties, then this call does nothing.
     **/
    public void integrateWithSystemProperties() {
	if (list != System.getProperties()) {
	    load(System.getProperties());
	    setSystemProperties();
	}
    }

    /**
     * Formats all of the properties as a String.  This doesn't
     * include the applet parameters.
     **/
    public String toString() {
	java.io.StringWriter stringWriter = new java.io.StringWriter();
	grace.io.JavaPrintWriter writer =
	    new grace.io.JavaPrintWriter(stringWriter);
	writer.print("list", list);
	writer.close();
	return stringWriter.toString();
    }

    /**
     * Returns a list of the properties.  This doesn't include the
     * applet parameters.
     **/
    public java.util.Properties getJavaProperties() { return list; }

    /**
     * Sets the list of the properties maintained by this class.  This
     * doesn't include the applet parameters.
     **/
    public void set(java.util.Properties list) {
	this.list = list;
    }
}
