//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
package grace.util;

import java.util.Vector;
import java.net.MalformedURLException;

import gnu.regexp.RE;
import gnu.regexp.REException;

public class Registry {
    /**
     * This utility function simply parses and returns the file
     * portion (also called "the name") of the given url.  This is
     * like the java.net.URL.getFile() function but since Sun didn't
     * handle the "rmi:" protocol, it has to be done here.
     **/
    public static String getURLFile(String url) throws MalformedURLException {
	try {
	    int thirdSlashPos =
		url.indexOf(
		    '/', 1 + url.indexOf(
			'/', 1 + url.indexOf('/')));
	    return (url.length() > thirdSlashPos + 1) ?
		url.substring(thirdSlashPos + 1) : "";
	}
	catch (Exception e) {
	    throw new MalformedURLException("not enough slashes");
	}
    }

    /**
     * This utility function simply parses and returns the hostname
     * portion of the given url.  This is like the
     * java.net.URL.getHost() function but since Sun didn't handle the
     * "rmi:" protocol, it has to be done here.
     **/
    public static String getURLHost(String url) throws MalformedURLException {
	try {
	    int secondSlashPos = url.indexOf('/', 1 + url.indexOf('/'));
	    int thirdSlashPos = url.indexOf('/', 1 + secondSlashPos);
	    if (thirdSlashPos == -1) thirdSlashPos = url.length();
	    return (url.length() > secondSlashPos + 1) ?
		url.substring(secondSlashPos + 1, thirdSlashPos) : "";
	}
	catch (Exception e) {
	    throw new MalformedURLException("not enough slashes");
	}
    }


    /**
     * This utility function searches for matches to a URL of the
     * form: "rmi://hostname/regexp".  This means that a list the
     * names matching 'regexp' on the rmi registry localed at hostname
     * will be returned.  This uses getURLFile to determine the
     * regular expression part and getURLHost to find the hostname of
     * the registry.
     *
     * @see getURLFile()
     * @see getURLHost()
     **/
    public static String[] lookup(String regularExpressionURL)
	throws REException,
	java.rmi.RemoteException,
	java.rmi.UnknownHostException,
	java.net.MalformedURLException
    {
	System.out.println();
	System.out.println(">>>>>>>>>> url=" + regularExpressionURL);
	System.out.println(">>>>>>>>>> host=" + getURLHost(regularExpressionURL));
	System.out.println();
	String[] entries = java.rmi.Naming.list(
	    "rmi://" + getURLHost(regularExpressionURL));


	Vector matches = new Vector();
	RE expression = new RE(getURLFile(regularExpressionURL));
	for (int i = 0; i < entries.length; ++i) {
	    System.out.println(">>>>>>>>>> entry=" + entries[i]);
	    if (expression.isMatch(getURLFile(entries[i]))) {
		matches.addElement(entries[i]);
	    }
	}
	System.out.println(">>>>>>>>>> done");

	String[] result = new String[matches.size()];
	matches.copyInto(result);
	return result;
    }


    /**
     * Simple test function
     **/
    public static void main(String[] args) {
	String url = "rmi://host.host.host/MyServer.MyServer";

	try {
	    System.out.println("getURLFile() " +
		((getURLFile(url).equals("MyServer.MyServer"))
		? "passed" : "failed"));
	    System.out.println("getURLHost() " +
		((getURLHost(url).equals("host.host.host"))
		? "passed" : "failed"));
	}
	catch (Exception e) { e.printStackTrace(); }
    }
}
