//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
//
// Copyright (C) 1999 Grace Software
//
// The contents of this file are subject to the Grace Software Public
// License Version 1.0 (the "GracePL"); you may not use this file
// except in compliance with the GracePL. You may obtain a copy of the
// GracePL at http://www.homestead.com/javalog/files/license.html
//
// Software distributed under the GracePL is distributed on an "AS IS"
// basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
// the GracePL for the specific language governing rights and
// limitations under the GracePL.
//
// The Original Code is Grace Software's JavaLog code, released March
// 21, 1999.
//
// The Initial Developer of the Original Code is Grace Software.
// Portions created by Grace Software are Copyright (C) 1999 Grace
// Software. All Rights Reserved.
//
// Contributor(s):
//
package grace.util;

import java.util.Date;
import java.text.DecimalFormat;

/**
 * This is a simple, general purpose timer useful for finding the
 * duration of time between a sequence of start and stop events.  The
 * typical use of this timer is to create it, start it, stop it, and
 * read the duration of time that has elapsed between the start and
 * stop.  However, the timer will accumate all times between
 * subsequent start and stop events so the timer can be started and
 * stopped manny times before it is zeroed.  Also, the timer does not
 * need to be stopped in order to read the duration.
 **/
public class Timer {
    private Date start;
    private long duration = 0;
    private static DecimalFormat hoursFormat = new DecimalFormat("00");
    private static DecimalFormat minutesFormat = hoursFormat;
    private static DecimalFormat secondsFormat = new DecimalFormat("00.###");


    public Timer() {}

    public Timer(Timer toCopy) {
	if (toCopy != null) {
	    if (toCopy.start == null) start = null;
	    else start = new Date(toCopy.start.getTime());

	    duration = toCopy.duration;
	}
    }
    
    public void start() {
	if (start == null) start = new Date();
    }

    public void stop() {
	if (start != null) {
	    Date stop = new Date();
	    duration += stop.getTime() - start.getTime();
	    start = null;
	}
    }

    public boolean isStopped() {
	return start == null;
    }

    public void zero() {
	duration = 0;
	if (start != null) start = new Date();
    }

    public long durationInMilliSeconds() {
	if (start == null) return duration;
	else {
	    Date now = new Date();
	    return now.getTime() - start.getTime() + duration;
	}
    }

    public double duration() {
	return durationInMilliSeconds() / 1000.0;
    }

    public String toString() {
	double seconds = duration();
	StringBuffer result = new StringBuffer();

	if (!isStopped()) result.append("*");

	if (seconds > 60.0) {
	    long minutes = (long) (seconds / 60.0);
		
	    if (minutes > 60) {
		long hours = minutes / 60;
		result.append(hoursFormat.format(hours) + ":");
	    }

	    result.append(minutesFormat.format(minutes) + ":");
	    result.append(secondsFormat.format(seconds));
	}
	else {
	    result.append("" + seconds);
	}

	return result.toString();
    }


    public static void main(String[] args) {
	Timer t = new Timer();

	t.start();

	try { Thread.sleep(1230); }
	catch (Exception e) {}

	System.out.println("should be about 1.23 :" + t);

	try { Thread.sleep(100); }
	catch (Exception e) {}

	t.stop();
	System.out.println("should be about 1.33 : " + t);

	try { Thread.sleep(100); }
	catch (Exception e) {}

	System.out.println("should still be about 1.33 : " + t);
	try { Thread.sleep(100); }
	catch (Exception e) {}
	System.out.println("should still be about 1.33 : " + t);


	t.start();

	try { Thread.sleep(100); }
	catch (Exception e) {}

	System.out.println("should be about 1.43 : " + t);
	t.zero();

	try { Thread.sleep(100); }
	catch (Exception e) {}

	System.out.println("should be about 0.1 : " + t);
    }
}
    
    
